%Function to calculate how many components a link has, when given as a
%braid representation.

function f=link_number(x)

%Here we follow each strand in turn through the braid and see where its end
%position is.  If we are following strand i and we come across a crossing
%with modulus i, then the strand becomes strand i+1; if we come across a
%crossing with modulus i-1, then the strand moves up to become strand i-1.
%With a crossing of any other modulus the strand is unaffected.
%The vector y will record this information, and will be called the braid
%permutation.
y=[];
for n=1:max(abs(x))+1
    m=n;
    for i=1:length(x)
    	if abs(x(i))==m
	        m=m+1;
        elseif abs(x(i))==m-1
	        m=m-1;
	    else m=m;
        end
    end
    y(n)=m;
end

%The number of links in the braid corresponds to the number of cycles in
%the braid permutation y.  We follow the permutations in y until we have
%reached the number we started with, at which point we know we have found a
%cycle.  We also know that we need not check any of the numbers in the
%cycle as a possible starting point for another cycle.
%For example, suppose y=[3 1 2 5 4]. If we start at the first entry in this
%vector we see that strand 1 becomes 3, then strand 3 becomes strand 2, and
%strand 2 becomes strand 1, which is where we started.  This gives us a cycle 
%1 -> 3 -> 2 -> 1.  The other cycle is 4 -> 5 -> 4.  Since there are two
%cycles, this is a two-component link.

checked=zeros(1,length(y));
count=0;
for i=1:length(y)
    if checked(i)==0
        checked(i)=1;
        k=y(i);
        while k~=i
            checked(k)=1;
            k=y(k);
        end 
        count=count+1;
    end
end
f=count;